@@ -22,6 +22,7 @@ class Bancor3(EnvBase):
     name = "Bancor3 Environment"
     block_num = 0
     ema = {
+        'TKN': {'block_num': [0], 'ema': [0]},
         'BNT': {'block_num': [0], 'ema': [0]},
         'wBTC': {'block_num': [0], 'ema': [0]},
         'ETH': {'block_num': [0], 'ema': [0]},
@@ -37,16 +38,56 @@ class Bancor3(EnvBase):
             exit_fee=.0025,
             cooldown_period=7,
             bnt_funding_limit=100000,
-            external_price_feed={},
-            pool_fees={},
+            external_price_feed={
+                        "TKN": {'block_num': [0, 1, 2, 3],
+                                'price_usd': [2.50, 2.49, 2.51, 2.50]},
+                        "BNT": {'block_num': [0, 1, 2, 3],
+                                'price_usd': [2.50, 2.49, 2.51, 2.50]},
+                        "LINK": {'block_num': [0, 1, 2, 3],
+                                'price_usd': [15.35, 15.20, 15.11, 15.00]},
+                        "ETH": {'block_num': [0, 1, 2, 3],
+                                'price_usd': [2550.00, 2400.00, 2450.00, 2500.00]},
+                        "wBTC": {'block_num': [0, 1, 2, 3],
+                                'price_usd': [40123.23, 40312.21, 40111.11, 40000.00]}
+            },
+            pool_fees={
+                "TKN": 0.01,
+                "BNT": 0.01,
+                "LINK": 0.01,
+                "ETH": 0.01,
+                "wBTC": 0.01
+            },
             whitelisted_tokens=[],
             bootstrapped_tokens=[],
+            spot_prices={
+                "TKN": {'block_num': [0],
+                        'price_usd': [2.50]},
+                "BNT": {'block_num':[0],
+                        'price_usd':[2.50]},
+                "LINK": {'block_num':[0],
+                        'price_usd':[15.00]},
+                "ETH": {'block_num':[0],
+                        'price_usd':[2500.00]},
+                "wBTC": {'block_num':[0],
+                        'price_usd':[40000.00]}
+            },
+            vortex_rates={
+                "TKN": 0.2,
+                "BNT": 0.2,
+                "LINK": 0.2,
+                "ETH": 0.2,
+                "wBTC": 0.2
+            },
             dao_msig_initialized_pools=[],
             verbose=True,
             **kwargs
     ):
 
+        super().__init__()
+
         self.set_param(alpha=alpha,
+                       spot_prices=spot_prices,
+                       vortex_rates=vortex_rates,
                        block_num=block_num,
                        min_liquidity_threshold=min_liquidity_threshold,
                        bnt_funding_limit=bnt_funding_limit,
@@ -60,8 +101,6 @@ class Bancor3(EnvBase):
                        dao_msig_initialized_pools=dao_msig_initialized_pools,
                        verbose=verbose)
 
-        self._spot_prices = {}
-        super().__init__()
         for tkn in self.available_liquidity_ledger:
             self.available_liquidity_ledger[tkn]['funding_limit'][-1] = bnt_funding_limit
 
@@ -214,18 +253,8 @@ class Bancor3(EnvBase):
             bnt_trading_liquidity, tkn_trading_liquidity, tkn_out, tkn_fee, vortex_fee = operation(
                 a, d, b, x, e)
 
-            self.available_liquidity_ledger[tkn_a]['BNT'].append(bnt_trading_liquidity)
-            self.available_liquidity_ledger[tkn_a][tkn_a].append(tkn_trading_liquidity)
-            self.available_liquidity_ledger[tkn_a]['funding_limit'].append(
-                self.available_liquidity_ledger[tkn_a]['funding_limit'][-1])
-            self.available_liquidity_ledger[tkn_a]['block_num'].append(
-                self.available_liquidity_ledger[tkn_a]['block_num'][-1])
-            for key in self.available_liquidity_ledger:
-                if key != tkn_a:
-                    self.available_liquidity_ledger[key]['funding_limit'].append(self.available_liquidity_ledger[key]['funding_limit'][-1])
-                    self.available_liquidity_ledger[key]['block_num'].append(self.available_liquidity_ledger[key]['block_num'][-1])
-                    self.available_liquidity_ledger[key]['BNT'].append(self.available_liquidity_ledger[key]['BNT'][-1])
-                    self.available_liquidity_ledger[key][key].append(self.available_liquidity_ledger[key][key][-1])
+            if self.is_within_ema_tolerance(tkn_a):
+                self.update_available_liquidity_ledger(tkn_a, bnt_trading_liquidity, tkn_trading_liquidity)
 
             self.add_tkn_to_vault(tkn1, x)
             self.add_tkn_to_vault(tkn2, -tkn_out)
@@ -253,28 +282,12 @@ class Bancor3(EnvBase):
             bnt_source_trading_liquidity, tkn_source_trading_liquidity, bnt_destination_trading_liquidity, tkn_destination_trading_liquidity, tkn_out, bnt_fee, tkn_fee, vortex_fee = self.tokenomics.handle_trade_tkn_to_tkn(
                 a1, d1, b1, a2, d2, b2, x, e)
 
+
             if self.is_within_ema_tolerance(tkn1):
-                self.available_liquidity_ledger[tkn1]['BNT'].append(bnt_source_trading_liquidity)
-                self.available_liquidity_ledger[tkn1][tkn1].append(tkn_source_trading_liquidity)
-                self.available_liquidity_ledger[tkn1]['funding_limit'].append(
-                    self.available_liquidity_ledger[tkn1]['funding_limit'][-1])
-                self.available_liquidity_ledger[tkn1]['block_num'].append(
-                    self.available_liquidity_ledger[tkn1]['block_num'][-1])
+                self.update_available_liquidity_ledger(tkn1, bnt_source_trading_liquidity, tkn_source_trading_liquidity)
 
             if self.is_within_ema_tolerance(tkn2):
-                self.available_liquidity_ledger[tkn2]['BNT'].append(bnt_destination_trading_liquidity)
-                self.available_liquidity_ledger[tkn2][tkn2].append(tkn_destination_trading_liquidity)
-                self.available_liquidity_ledger[tkn2]['funding_limit'].append(
-                    self.available_liquidity_ledger[tkn2]['funding_limit'][-1])
-                self.available_liquidity_ledger[tkn2]['block_num'].append(
-                    self.available_liquidity_ledger[tkn2]['block_num'][-1])
-
-            for key in self.available_liquidity_ledger:
-                if key not in [tkn1, tkn2]:
-                    self.available_liquidity_ledger[key]['funding_limit'].append(self.available_liquidity_ledger[key]['funding_limit'][-1])
-                    self.available_liquidity_ledger[key]['block_num'].append(self.available_liquidity_ledger[key]['block_num'][-1])
-                    self.available_liquidity_ledger[key]['BNT'].append(self.available_liquidity_ledger[key]['BNT'][-1])
-                    self.available_liquidity_ledger[key][key].append(self.available_liquidity_ledger[key][key][-1])
+                self.update_available_liquidity_ledger(tkn2, bnt_destination_trading_liquidity, tkn_destination_trading_liquidity)
 
             self.add_tkn_to_vault(tkn1, x)
             self.add_tkn_to_vault(tkn2, -tkn_out)
@@ -290,6 +303,9 @@ class Bancor3(EnvBase):
                   f'bnt_fee={bnt_fee}, \n',
                   f'tkn_fee={tkn_fee}, \n',
                   f'vortex_fee={vortex_fee} \n')
+
         self.check_pool_shutdown()
+        # self.get_spot_rate(tkn1, update_spot_prices=True)
+        # self.get_spot_rate(tkn2, update_spot_prices=True)
         return tkn_out
 
