@@ -22,33 +22,14 @@ class EnvBase(ABC):
         random_seed (int): Seed for the random number generator. Defaults to 0.
     """
     metadata = {'render.modes': []}
-    external_price_feed = {
-        "BNT": {
-            'blockNo.': [0, 1, 2, 3],
-            'priceUSD': [2.50, 2.49, 2.51, 2.50]
-        },
-        "LINK": {
-            'blockNo.': [0, 1, 2, 3],
-            'priceUSD': [15.35, 15.20, 15.11, 15.00]
-        },
-        "ETH": {
-            'blockNo.': [0, 1, 2, 3],
-            'priceUSD': [2550.00, 2400.00, 2450.00, 2500.00]
-        }
-        ,
-        "wBTC": {
-            'blockNo.': [0, 1, 2, 3],
-            'priceUSD': [40123.23, 40312.21, 40111.11, 40000.00]
-        }
-    }
+
 
     def __init__(
         self,
         # observation_space=observation_space,
-        external_price_feed=external_price_feed,
     ):
         # self.observation_space = observation_space
-        self._external_price_feed = external_price_feed
+        # self._external_price_feed = external_price_feed
         # self.logger = get_logger(f"{__name__}")
 
         # create a vault
@@ -199,9 +180,9 @@ class EnvBase(ABC):
     def spot_prices(self, updated_spot_prices: dict):
         self._spot_prices = updated_spot_prices
         exchange_rates = {}
-        bnt_price = self._spot_prices['BNT']
+        bnt_price = self._spot_prices['BNT']['price_usd'][-1]
         for tkn in self._spot_prices:
-            tkn_price = self._spot_prices[tkn]
+            tkn_price = self._spot_prices[tkn]['price_usd'][-1]
             exchange_rates[tkn] = tkn_price / bnt_price
         self.exchange_rates = exchange_rates
         self.set_whitelisted_tokens(whitelisted_tokens=[tkn for tkn in updated_spot_prices])
@@ -218,7 +199,6 @@ class EnvBase(ABC):
 
     def set_param(self, is_solidity=None,
                         block_num=None,
-                        actions_per_block=None,
                         min_liquidity_threshold=None,
                         bnt_funding_limit=None,
                         alpha=None,
@@ -253,8 +233,6 @@ class EnvBase(ABC):
         """
         if block_num is not None:
             self.block_num = block_num
-        if actions_per_block is not None:
-            self.actions_per_block = actions_per_block
         if min_liquidity_threshold is not None:
             self.min_liquidity_threshold = min_liquidity_threshold
         if bnt_funding_limit is not None:
@@ -349,7 +327,7 @@ class EnvBase(ABC):
             bnTKN_value = 1
         return bnTKN_value
 
-    def get_spot_rate(self, tkn):
+    def get_spot_rate(self, tkn, update_spot_prices=False):
         """Gets the spot rate in units of BNT/TKN as determined by the trading liquidity balances of the pool
 
         Args:
@@ -366,9 +344,15 @@ class EnvBase(ABC):
                 spot_rate = BNT_amt / TKN_amt
             else:
                 spot_rate = self.exchange_rates[tkn]
+
+        if update_spot_prices:
+            spot_prices = self.spot_prices
+            spot_prices[tkn]['block_num'].append(self.block_num)
+            spot_prices[tkn]['price_usd'].append(spot_rate)
+            self.spot_prices = spot_prices
         return spot_rate
 
-    def get_bnt_rate(self, tkn):
+    def get_bnt_rate(self, tkn) -> float:
         """Gets the rate in units of TKN/BNT as determined by the external price feed
 
         Args:
@@ -376,8 +360,8 @@ class EnvBase(ABC):
         Returns:
             bnt_rate (int or float or decimal): bnt rate in units of TKN/BNT
         """
-        bnt_price = self.external_price_feed['BNT']['priceUSD'][-1]
-        tkn_price = self.external_price_feed[tkn]['priceUSD'][-1]
+        bnt_price = self.external_price_feed['BNT']['price_usd'][-1]
+        tkn_price = self.external_price_feed[tkn]['price_usd'][-1]
         bnt_rate = tkn_price / bnt_price
         return bnt_rate
 
@@ -446,7 +430,8 @@ class EnvBase(ABC):
             l = ema[token]['block_num'] # gets list of the current block history for any token
             break
 
-        ema[tkn] = {'block_num': [x for x in l], 'ema': [self.exchange_rates[tkn] for x in range(len(l))]}
+        # ema[tkn] = {'block_num': [x for x in l], 'ema': [self.exchange_rates[tkn] for x in range(len(l))]}
+
         if tkn in self.ema:
             if ema[tkn]['block_num'][-1] < self.block_num:
                 ema[tkn]['block_num'].append(self.block_num)
@@ -476,11 +461,14 @@ class EnvBase(ABC):
         return result
 
     def add_tkn_to_vault(self, tkn, x):
-        '''
-        Creates a new row with the same block_num number inside the self.vault_ledger dictionary.
-        Adds x TKN to the appropriate list; all other TKN balances, and the block_num number are carried forward.
-        Returns nothing.
-        '''
+        """Adds x tkn to the self.vault_ledger dictionary with the same block_num number.
+
+        Args:
+            tkn (str): The tickername of the token.
+            x (int or float or decimal): The amount of tkn.
+        Returns:
+            self
+        """
         for key in self.vault_ledger:
             if key == tkn:
                 self.vault_ledger[key].append(self.vault_ledger[key][-1] + x)
@@ -489,11 +477,14 @@ class EnvBase(ABC):
             self.vault_ledger['block_num'][-1] = self.block_num
 
     def add_tkn_to_staking_ledger(self, tkn, x):
-        '''
-        Creates a new row with the same block_num number inside the self.staking_ledger dictionary.
-        Adds x TKN to the appropriate list; all other TKN balances, and the block_num number are carried forward.
-        Returns nothing.
-        '''
+        """Adds x tkn to the self.staking_ledger dictionary with the same block_num number.
+
+        Args:
+            tkn (str): The tickername of the token.
+            x (int or float or decimal): The amount of tkn.
+        Returns:
+            self
+        """
         for key in self.staking_ledger:
             if key == tkn:
                 self.staking_ledger[key].append(self.staking_ledger[key][-1] + x)
@@ -503,8 +494,10 @@ class EnvBase(ABC):
 
     def add_tkn_to_vortex_ledger(self, x):
         """Adds TKN to the vortex ledger.
+
         Args:
-            x (int or float or decimal): The value of BNT to add to the vortex
+            x (int or float or decimal): The value of BNT to add to the vortex.
+
         Returns:
             self
         """
@@ -513,9 +506,11 @@ class EnvBase(ABC):
 
     def add_tkn_to_available_liquidity_ledger(self, tkn, x):
         """Adds TKN to the available liquidity ledger.
+
         Args:
             tkn (str): The tickername of the token.
             x (int or float or decimal): The value of liquidity to add.
+
         Returns:
             self
         """
@@ -542,10 +537,12 @@ class EnvBase(ABC):
 
     def mint_bntkn(self, tkn, x=None, bnt=None):
         """Adds TKN to the available liquidity ledger.
+
         Args:
             tkn (str): The tickername of the token.
             x (int or float or decimal): The value of liquidity to add. (default=None)
             bnt (int or float or decimal): The value of bnt liquidity to add. (default=None)
+
         Returns:
             y (int): The number of pool tokens issued to the user.
         """
@@ -559,7 +556,7 @@ class EnvBase(ABC):
                 self.pool_token_supply_ledger[contract]['supply'].append(self.pool_token_supply_ledger[contract]['supply'][-1])
         return (y)
 
-    def check_pool_shutdown(self):
+    def check_pool_shutdown(self, warning_only=True):
         """Checks if any whitelisted tokens meet the pool-shutdown criterea, and resets the DOAmsig state if yes.
 
         Returns:
@@ -568,21 +565,22 @@ class EnvBase(ABC):
         dao_msig_initialized_pools = self.dao_msig_initialized_pools
         for TKN in self.whitelisted_tokens:
             BNTperTKN = self.get_bnt_rate(TKN)
-            if self.vault_ledger[TKN][-1] * BNTperTKN < self.min_liquidity_threshold and (TKN in self.dao_msig_initialized_pools):
-
-                dao_msig_initialized_pools = [token for token in dao_msig_initialized_pools if token != TKN]
-                self.available_liquidity_ledger[TKN]['BNT'].append(0)
-                self.available_liquidity_ledger[TKN][TKN].append(0)
-                self.available_liquidity_ledger[TKN]['funding_limit'].append(self.bnt_funding_limit)
-                self.available_liquidity_ledger[TKN]['block_num'].append(
-                    self.available_liquidity_ledger[TKN]['block_num'][-1])
-                self.dao_msig_initialized_pools = dao_msig_initialized_pools
+            if float(self.vault_ledger[TKN][-1]) * float(BNTperTKN) < self.min_liquidity_threshold and (TKN in self.dao_msig_initialized_pools):
+                # if not warning_only:
+                #     dao_msig_initialized_pools = [token for token in dao_msig_initialized_pools if token != TKN]
+                #     self.available_liquidity_ledger[TKN]['BNT'].append(0)
+                #     self.available_liquidity_ledger[TKN][TKN].append(0)
+                #     self.available_liquidity_ledger[TKN]['funding_limit'].append(self.bnt_funding_limit)
+                #     self.available_liquidity_ledger[TKN]['block_num'].append(
+                #         self.available_liquidity_ledger[TKN]['block_num'][-1])
+                #     self.dao_msig_initialized_pools = dao_msig_initialized_pools
 
                 if self.verbose:
                     print(
                         f"WARNING POOL SHUTDOWN!!! \n",
+                        f"warning_only={warning_only} \n",
                         f"pool={TKN} \n",
-                        f"TKN Liquidity <= Minimum Pool Liquidity, resetting pool to pre-DAOmsig state"
+                        f"TKN Liquidity <= Minimum Pool Liquidity, if not warning_only then will reset pool to pre-DAOmsig state"
                     )
 
     def check_bootstrapped_tokens(self):
@@ -690,6 +688,32 @@ class EnvBase(ABC):
         self.params_set = True
         return self.validated_params.a, self.validated_params.b, self.validated_params.c, self.validated_params.e, self.validated_params.m, self.validated_params.n, self.validated_params.x, self.validated_params.w
 
+    def update_available_liquidity_ledger(self, tkn, bnt_trading_liquidity, tkn_trading_liquidity):
+        """Updates available liquidity for swaps
+
+        Args:
+            tkn (str): The tickername of the token.
+            bnt_trading_liquidity (int or float or decimal): The value of liquidity to add to bnt.
+            tkn_trading_liquidity (int or float or decimal): The value of liquidity to add to tkn.
+
+        Returns:
+            self
+        """
+        self.available_liquidity_ledger[tkn]['BNT'].append(bnt_trading_liquidity)
+        self.available_liquidity_ledger[tkn][tkn].append(tkn_trading_liquidity)
+        self.available_liquidity_ledger[tkn]['funding_limit'].append(
+            self.available_liquidity_ledger[tkn]['funding_limit'][-1])
+        self.available_liquidity_ledger[tkn]['block_num'].append(
+            self.available_liquidity_ledger[tkn]['block_num'][-1])
+        for key in self.available_liquidity_ledger:
+            if key != tkn:
+                self.available_liquidity_ledger[key]['funding_limit'].append(
+                    self.available_liquidity_ledger[key]['funding_limit'][-1])
+                self.available_liquidity_ledger[key]['block_num'].append(
+                    self.available_liquidity_ledger[key]['block_num'][-1])
+                self.available_liquidity_ledger[key]['BNT'].append(self.available_liquidity_ledger[key]['BNT'][-1])
+                self.available_liquidity_ledger[key][key].append(self.available_liquidity_ledger[key][key][-1])
+
     def update_ledger_balances(self, tkn, p, q, r, s, t, u, a, b, c, e, m, n, x, w, case, event_type='unstake_tkn'):
         """Updates the ledger balances for withdraw actions
 
